/*
 * AAHelper - An AALib Helper Libary
 * (C) Gary Briggs 2003-4
 *
 * AAHelper is free software; you can redistribute it and/or modify
 * it under the  terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * Library General Public License for more details.
 */ 

#include <stdlib.h>
#include <math.h>
#include <aalib.h>
#include "./aa_helperint.h"
#include "./aa_helper.h"


void aa_drawpoly(aa_context *context, int numpoints,
				struct aa_point pointlist[], int color,
				int hints) {
	if(numpoints < 3) {
		/* Let us not draw this. It is a silly polygon */
		return;
	}

	if(hints & AA_HINT_CONVEX) {
		aa_drawconvexpoly(context,numpoints,pointlist,color);
	} else if (hints & AA_HINT_NONCONVEX) {
		aa_drawnonconvexpoly(context,numpoints,pointlist,color);
	} else {
		aa_drawcomplexpoly(context,numpoints,pointlist,color);
	}
}

/* Abrash Black Book, Chapter 38.
 Somewhat modified and stuff moved about and renamed so I can understand it */

/* Thoughts on Abrash:
	1) His macro aproach for iterating over stuff is ugly.
	2) In fact, most of his loops are ugly.
	2) By changing the data structures somewhat, you can ignore the
		whole render-ends-of-lines-twice problem.
		You lose a comparison-and-assignment, then gain a
		garaunteed-assignment.
	3) This is aalib. A bit of popping is normal.
*/


void aa_drawconvexpoly(aa_context *context, int numpoints,
				struct aa_point pointlist[], int color) {
	int i,j,k;
	int MaxY, MinY;
	int FirstTop, LastTop, TmpTop;
	int switchtmp;
	int leftedgedir;

	int *leftedgelist, *rightedgelist;
		/* List of left and right points to render to & from */
	int currentleftindex, currentrightindex;
		/* Index of which point we're on, going round the poly */

	leftedgedir = -1; /* Means that left edge goes down the vertex list */
	MaxY = pointlist[0].y;
	MinY = MaxY;

	/* Find top Y value in polygon */
	for (i=1;i<numpoints;i++) {
		if(pointlist[i].y > MaxY) {
			MaxY = pointlist[i].y;
			TmpTop = i; /* Not necessarily last top */
		} else if(pointlist[i].y < MinY) {
			MinY = pointlist[i].y;
		}
	}
	if(MaxY - MinY <= 1) {
		/* Let us not draw this. It, also, is a silly polygon */
		return;
	}

	/* Find where the top starts & ends. */
	for(i=TmpTop; i<TmpTop + numpoints; i++) {
		j = i % numpoints;
		if(pointlist[j].y == MaxY) {
			LastTop = j;
		} else if(pointlist[j].y < MaxY) {
			break;
		}
	}
	for(i=TmpTop+numpoints; i>=TmpTop; i-- ) {
		j = i % numpoints;
		if(pointlist[j].y == MaxY) {
			FirstTop = j;
		} else if(pointlist[j].y < MaxY) {
			break;
		}
	}

	if(LastTop != FirstTop) {
		if(pointlist[LastTop].x < pointlist[FirstTop].x) {
			leftedgedir = 1;
			switchtmp = FirstTop;
			FirstTop = LastTop;
			LastTop = switchtmp;
		}
	} else {
		if(pointlist[FirstTop+1].x < pointlist[FirstTop-1].x) {
			leftedgedir = 1;
		}
	}

	leftedgelist = (int *)malloc((size_t)(sizeof(int) * (MaxY - MinY)));
	if(leftedgelist == NULL) {
		return;
	}
	rightedgelist = (int *)malloc((size_t)(sizeof(int) * (MaxY - MinY)));
	if(rightedgelist == NULL) {
		return;
	}

	j = FirstTop;
	do {
		k = (j+leftedgedir)%numpoints;
		if(k<0) k+= numpoints;

		currentleftindex = pointlist[k].y - MinY;
		aa_scanedge(
			pointlist[k].x,
			pointlist[k].y,
			pointlist[j].x,
			pointlist[j].y,
			leftedgelist+currentleftindex);

		j = (j+leftedgedir)%numpoints;
		if(j<0) j+= numpoints;
	} while (pointlist[k].y > MinY);

	j = LastTop;
	do {
		k = (j-leftedgedir)%numpoints;
		if(k<0) k+= numpoints;

		currentrightindex = pointlist[k].y - MinY;
		aa_scanedge(
			pointlist[k].x,
			pointlist[k].y,
			pointlist[j].x,
			pointlist[j].y,
			rightedgelist+currentrightindex);

		j = (j-leftedgedir)%numpoints;
		if(j<0) j+= numpoints;
	} while (pointlist[k].y > MinY);

	aa_fastrender(context, 0, 0,
		aa_scrwidth(context), aa_scrheight(context));
	aa_flush(context);

	for(i = 0; i < MaxY-MinY; i++) {
		aa_drawline(context, leftedgelist[i], i+MinY,
					rightedgelist[i], i+MinY,
				color);
	}

	free((void *)leftedgelist);
	free((void *)rightedgelist);
}


void aa_drawnonconvexpoly(aa_context *context, int numpoints,
				struct aa_point pointlist[], int color) {
	aa_drawcomplexpoly(context, numpoints, pointlist, color);
}

void aa_drawcomplexpoly(aa_context *context, int numpoints,
				struct aa_point pointlist[], int color) {
	struct aa_EdgeState *EdgeTableBuffer;
	int CurrentY;

	struct aa_EdgeState *aa_GETPtr, *aa_AETPtr;

	aa_GETPtr = NULL;
	aa_AETPtr = NULL;

	/* Get enough memory to store the entire edge table */
	EdgeTableBuffer = (struct aa_EdgeState *)
		(malloc(sizeof(struct aa_EdgeState) *numpoints));
	if (EdgeTableBuffer == NULL)
		return;  /* couldn't get memory for the edge table */

	/* Build the global edge table */
	aa_BuildGET(pointlist, numpoints, EdgeTableBuffer, 0, 0, &aa_GETPtr);
	/* Scan down through the polygon edges, one scan line at a time,
	so long as at least one edge remains in either the GET or AET */
	CurrentY = aa_GETPtr->StartY; /* start at the top polygon vertex */
	while ((aa_GETPtr != NULL) || (aa_AETPtr != NULL)) {
		aa_MoveXSortedToAET(CurrentY, &aa_GETPtr, &aa_AETPtr);
			/* update AET for this scan line */
		aa_ScanOutAET(context, CurrentY, color, &aa_AETPtr);
			/* draw this scan line from AET */
		aa_AdvanceAET(&aa_AETPtr);
			/* advance AET edges 1 scan line */
		aa_XSortAET(&aa_AETPtr);
			/* resort on X */
		CurrentY++;
			/* advance to the next scan line */
	}
	/* Release the memory we've allocated and we're done */
	free(EdgeTableBuffer);
}


/* populate pointlist with x co-ordinates for each y value */

void aa_scanedge(int x0, int y0, int x1, int y1, int *pointlist) {
	/* Maybe we could speed this up with bresenham's? */
	int y, dx, dy;
	float inverseslope;
	dx = x1 - x0;
	dy = y1 - y0;

	inverseslope = (float)dx / (float)dy;

	for(y=y0; y<=y1; y++) {
		pointlist[y-y0] = x0 + (int)ceil((double)(inverseslope * (y-y0)));
	}
}

/* draw a line into a buffer that's width w */

/* Bresenham's, flagrantly lifted from aadrawline, flagrantly lifted
from Abrash. */
void aa_tmpbuff_bres(int x0, int y0, int x1, int y1, int w, char *buffer) {
	int xlen, ylen;
	int i;
	int tmp;

	if(y0 == y1) {
		/* It's a flat line, which we can optimise a little :-) */
		if(x0 > x1) {
			tmp = x0;
			x0 = x1;
			x1 = tmp;
		}

		for(i=x0;i<=x1;i++) {
			buffer[y0*w+i]++;
		}
		return;
	}

	if(y0>y1) {
		tmp = y0;
		y0 = y1;
		y1 = tmp;
		tmp = x0;
		x0 = x1;
		x1 = tmp;
	}
	xlen = x1 - x0;
	ylen = y1 - y0;

	if ( xlen > 0 ) {
		if ( xlen > ylen ) {
			aa_tmpbuff_octant0(x0, y0, xlen, ylen, w, 1, buffer);
		} else {
			aa_tmpbuff_octant1(x0, y0, xlen, ylen, w, 1, buffer);
		}
	} else {
		xlen = -xlen;
		if ( xlen > ylen ) {
			aa_tmpbuff_octant0(x0, y0, xlen, ylen, w, -1, buffer);
		} else {
			aa_tmpbuff_octant1(x0, y0, xlen, ylen, w, -1, buffer);
		}
	}
}

/* These two octant things will just increment the point at that position
in the buffer. Color is irrelevant */

void aa_tmpbuff_octant0(int x0, int y0,
			int xlen, int ylen,
			int w, int xdir, char *buffer) {
	int ylenX2;
	int ylenminusxlenX2;
	int errorterm;

	ylenX2 = ylen * 2;
	ylenminusxlenX2 = ylenX2 - (int)(2 * xlen);
	errorterm = ylenX2 - (int)xlen;

	buffer[y0*w+x0]++;
	while (xlen--) {
		if ( errorterm >= 0 ) {
			y0++;
			errorterm += ylenminusxlenX2;
		} else {
			errorterm += ylenX2;
		}
		x0 += xdir;
		buffer[y0*w+x0]++;
	}
}

void aa_tmpbuff_octant1(int x0, int y0,
			int xlen, int ylen,
			int w, int xdir, char *buffer) {
	int xlenX2;
	int xlenminusylenX2;
	int errorterm;

	xlenX2 = xlen * 2;
	xlenminusylenX2 = xlenX2 - (int)(2 * ylen);
	errorterm = xlenX2 - (int)ylen;

	buffer[y0*w+x0]++;
	while (ylen--) {
		if ( errorterm >= 0 ) {
			x0+=xdir;
			errorterm += xlenminusylenX2;
		} else {
			errorterm += xlenX2;
		}
		y0++;
		buffer[y0*w+x0]++;
	}
}


/* From abrash */
void aa_BuildGET(struct aa_point *VertexList, int numvertices,
    struct aa_EdgeState *NextFreeEdgeStruc, int XOffset, int YOffset,
	struct aa_EdgeState **aa_GETPtr) {

	int i, StartX, StartY, EndX, EndY, DeltaY, DeltaX, Width;
	struct aa_EdgeState *NewEdgePtr;
	struct aa_EdgeState *FollowingEdge, **FollowingEdgeLink;

	/* Scan through the vertex list and put all non-0-height edges into
	the GET, sorted by increasing Y start coordinate */
	*aa_GETPtr = NULL;    /* initialize the global edge table to empty */
	for (i = 0; i < numvertices; i++) {
		/* Calculate the edge height and width */
		StartX = VertexList[i].x + XOffset;
		StartY = VertexList[i].y + YOffset;
		/* The edge runs from the current point to the previous one */
		if (i == 0) {
			/* Wrap back around to the end of the list */
			EndX = VertexList[numvertices-1].x + XOffset;
			EndY = VertexList[numvertices-1].y + YOffset;
		} else {
			EndX = VertexList[i-1].x + XOffset;
			EndY = VertexList[i-1].y + YOffset;
		}
		/* Make sure the edge runs top to bottom */
		if (StartY > EndY) {
			SWAP(StartX, EndX);
			SWAP(StartY, EndY);
		}
		/* Skip if this can't ever be an active edge (has 0 height) */
		if ((DeltaY = EndY - StartY) != 0) {
			/* Allocate space for this edge's info, and fill in the
			structure */
			NewEdgePtr = NextFreeEdgeStruc++;
			NewEdgePtr->XDirection =   /* direction in which X moves */
			((DeltaX = EndX - StartX) > 0) ? 1 : -1;
			Width = abs(DeltaX);
			NewEdgePtr->X = StartX;
			NewEdgePtr->StartY = StartY;
			NewEdgePtr->Count = DeltaY;
			NewEdgePtr->ErrorTermAdjDown = DeltaY;
			if (DeltaX >= 0)  /* initial error term going L->R */
				NewEdgePtr->ErrorTerm = 0;
			else              /* initial error term going R->L */
				NewEdgePtr->ErrorTerm = -DeltaY + 1;
			if (DeltaY >= Width) {     /* Y-major edge */
				NewEdgePtr->WholePixelXMove = 0;
				NewEdgePtr->ErrorTermAdjUp = Width;
			} else {                   /* X-major edge */
				NewEdgePtr->WholePixelXMove =
					(Width / DeltaY) * NewEdgePtr->XDirection;
				NewEdgePtr->ErrorTermAdjUp = Width % DeltaY;
			}
			/* Link the new edge into the GET so that the edge list is
			still sorted by Y coordinate, and by X coordinate for all
			edges with the same Y coordinate */
			FollowingEdgeLink = aa_GETPtr;
			for (;;) {
				FollowingEdge = *FollowingEdgeLink;
				if ((FollowingEdge == NULL) ||
					(FollowingEdge->StartY > StartY) ||
					((FollowingEdge->StartY == StartY) &&
					(FollowingEdge->X >= StartX))) {
					NewEdgePtr->NextEdge = FollowingEdge;
					*FollowingEdgeLink = NewEdgePtr;
					break;
				}
				FollowingEdgeLink = &FollowingEdge->NextEdge;
			}
		}
	}
}

void aa_MoveXSortedToAET(int YToMove, struct aa_EdgeState **aa_GETPtr,
	struct aa_EdgeState **aa_AETPtr) {

	struct aa_EdgeState *AETEdge, **AETEdgePtr, *TempEdge;
	int CurrentX;

	/* The GET is Y sorted. Any edges that start at the desired Y
	coordinate will be first in the GET, so we'll move edges from
	the GET to AET until the first edge left in the GET is no longer
	at the desired Y coordinate. Also, the GET is X sorted within
	each Y coordinate, so each successive edge we add to the AET is
	guaranteed to belong later in the AET than the one just added */
	AETEdgePtr = aa_AETPtr;
	while ((*aa_GETPtr != NULL) && ((*aa_GETPtr)->StartY == YToMove)) {
		CurrentX = (*aa_GETPtr)->X;
		/* Link the new edge into the AET so that the AET is still
		sorted by X coordinate */
		for (;;) {
			AETEdge = *AETEdgePtr;
			if ((AETEdge == NULL) || (AETEdge->X >= CurrentX)) {
				TempEdge = (*aa_GETPtr)->NextEdge;
				*AETEdgePtr = *aa_GETPtr;  /* link the edge into the AET */
				(*aa_GETPtr)->NextEdge = AETEdge;
				*aa_GETPtr = TempEdge;   /* unlink the edge from the GET */
				break;
			} else {
				AETEdgePtr = &AETEdge->NextEdge;
			}
		}
	}
}

void aa_ScanOutAET(aa_context *c, int YToScan, int color,
	struct aa_EdgeState **aa_AETPtr) {
	int LeftX;
	struct aa_EdgeState *CurrentEdge;

	/* Scan through the AET, drawing line segments as each pair of edge
	crossings is encountered. The nearest pixel on or to the right
	of left edges is drawn, and the nearest pixel to the left of but
	not on right edges is drawn */
	CurrentEdge = *aa_AETPtr;
	while (CurrentEdge != NULL) {
		LeftX = CurrentEdge->X;
		CurrentEdge = CurrentEdge->NextEdge;
		aa_drawline(c, LeftX, YToScan, CurrentEdge->X-1, YToScan, color);
		CurrentEdge = CurrentEdge->NextEdge;
	}
}

void aa_AdvanceAET(struct aa_EdgeState **aa_AETPtr) {
	struct aa_EdgeState *CurrentEdge, **CurrentEdgePtr;

	/* Count down and remove or advance each edge in the AET */
	CurrentEdgePtr = aa_AETPtr;
	while ((CurrentEdge = *CurrentEdgePtr) != NULL) {
	/* Count off one scan line for this edge */
		if ((--(CurrentEdge->Count)) == 0) {
			/* This edge is finished, so remove it from the AET */
			*CurrentEdgePtr = CurrentEdge->NextEdge;
		} else {
			/* Advance the edge's X coordinate by minimum move */
			CurrentEdge->X += CurrentEdge->WholePixelXMove;
			/* Determine whether it's time for X to advance one extra */
			if ((CurrentEdge->ErrorTerm +=
				CurrentEdge->ErrorTermAdjUp) > 0) {
				CurrentEdge->X += CurrentEdge->XDirection;
				CurrentEdge->ErrorTerm -= CurrentEdge->ErrorTermAdjDown;
			}
			CurrentEdgePtr = &CurrentEdge->NextEdge;
		}
	}
}

void aa_XSortAET(struct aa_EdgeState **aa_AETPtr) {
	struct aa_EdgeState *CurrentEdge, **CurrentEdgePtr, *TempEdge;
	int SwapOccurred;

	/* Scan through the AET and swap any adjacent edges for which the
	second edge is at a lower current X coord than the first edge.
	Repeat until no further swapping is needed */
	if (*aa_AETPtr != NULL) {
		do {
			SwapOccurred = 0;
			CurrentEdgePtr = aa_AETPtr;
			while ((CurrentEdge = *CurrentEdgePtr)->NextEdge != NULL) {
				if (CurrentEdge->X > CurrentEdge->NextEdge->X) {
					/* The second edge has a lower X than the first;
					swap them in the AET */
					TempEdge = CurrentEdge->NextEdge->NextEdge;
					*CurrentEdgePtr = CurrentEdge->NextEdge;
					CurrentEdge->NextEdge->NextEdge = CurrentEdge;
					CurrentEdge->NextEdge = TempEdge;
					SwapOccurred = 1;
				}
				CurrentEdgePtr = &(*CurrentEdgePtr)->NextEdge;
			}
		} while (SwapOccurred != 0);
	}
}
