/*
 * complexdemo - a demo of complex polygons in aahelper
 * (C) Gary Briggs 2003-4
 */ 

#include <aalib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <aalib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>


#include "aa_helper.h"

#define POLYPOINTS 5
#define STEPS 100
#define DELAY 5
struct aa_point star[POLYPOINTS];
struct aa_point pentagon[POLYPOINTS];
struct aa_point polygon[POLYPOINTS];
struct aa_point tmp;
aa_context *context;
aa_renderparams *params;

int main(int argc, char **argv) {

	int i, j, k;
	int unit_width, unit_height;
	struct timeval selecttime;
	struct aa_outputfile *output = NULL;

	/* INITIALISE */

	aa_parseoptions(NULL, NULL, &argc, argv);

	context = aa_autoinit(&aa_defparams);
	if (context == NULL) {
		printf("Failed to initialize aalib\n");
		exit(1);
	}
	aa_autoinitkbd(context, 0);
	params = aa_getrenderparams();
	aa_hidecursor(context);

	unit_width = aa_imgwidth(context)/6;
	unit_height = aa_imgheight(context)/5;

	star[4].x = unit_width * 3;
	star[4].y = unit_height * 4;

	star[3].x = unit_width * 4;
	star[3].y = unit_height * 1;

	star[2].x = unit_width * 1;
	star[2].y = unit_height * 3;

	star[1].x = unit_width * 5;
	star[1].y = unit_height * 3;

	star[0].x = unit_width * 2;
	star[0].y = unit_height * 1;

	pentagon[0].x = unit_width * 1;
	pentagon[0].y = unit_height * 3;

	pentagon[4].x = unit_width * 3;
	pentagon[4].y = unit_height * 5;

	pentagon[3].x = unit_width * 5;
	pentagon[3].y = unit_height * 3;

	pentagon[2].x = unit_width * 4;
	pentagon[2].y = unit_height * 1;

	pentagon[1].x = unit_width * 2;
	pentagon[1].y = unit_height * 1;

	if(getenv("AA_EXPORTAA") != NULL) {
		output = aa_openoutputfile(context, getenv("AA_EXPORTAA"));
	}

	for(k=0;k<POLYPOINTS;k++) {
		tmp.x = star[0].x;
		tmp.y = star[0].y;
		for(j=0;j<POLYPOINTS-1;j++) {
			star[j].x = star[j+1].x;
			star[j].y = star[j+1].y;
		}
		star[POLYPOINTS-1].x = tmp.x;
		star[POLYPOINTS-1].y = tmp.y;
		
		for(i=0;i<STEPS * 2;i++) {
			for(j=0;j<POLYPOINTS;j++) {
				if(i <= STEPS) {
					polygon[j].x =  star[j].x + (((float)i/STEPS) * (pentagon[j].x - star[j].x));
					polygon[j].y =  star[j].y + (((float)i/STEPS) * (pentagon[j].y - star[j].y));
				} else {
					polygon[j].x =  pentagon[j].x - (((float)(STEPS-i)/STEPS) * (star[j].x - pentagon[j].x));
					polygon[j].y =  pentagon[j].y - (((float)(STEPS-i)/STEPS) * (star[j].y - pentagon[j].y));
				}
			}

			selecttime.tv_sec = 0;
			selecttime.tv_usec = DELAY;
			select(0,NULL,NULL,NULL,&selecttime);

			aa_drawpoly(context, POLYPOINTS, polygon,  255, AA_HINT_COMPLEX);
			aa_render(context, params, 0, 0,
				aa_imgwidth(context), aa_imgheight(context));
			aa_writeframe(context, output);
			aa_flush(context);
			memset(context->imagebuffer,0,
				(size_t)(aa_imgwidth(context)*aa_imgheight(context)));

			if(aa_getkey(context, 0) == 'q') {
				aa_closeoutputfile(output);
				aa_close(context);
				return 0;
			}
		}
	}

	/* Shutdown */

	aa_closeoutputfile(output);
	aa_close(context);
	return 0;
}
