/*
 * aa2mpeg - a small tool to convert .aa to .mpeg using ffmpeg
 * (C) Gary Briggs 2003-4
 *
 * AAHelper is free software; you can redistribute it and/or modify
 * it under the  terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2.1 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT  ANY  WARRANTY;  without   even  the  implied  warranty  of
 * MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.   See the GNU
 * Library General Public License for more details.
 */ 

#include <aalib.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <aalib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>


#include "aa_helper.h"
#include "aa_helperint.h"
#include "avcodec.h"

int main(int argc, char **argv) {
	FILE *outputfile;
	unsigned char *attrbuf, *textbuf;
	__AA_CONST struct aa_font *font;
	size_t bufsize;
	struct aa_inputfile *inputfile;

	/* libavcodec stuff */
	AVCodec *codec;
	AVCodecContext *mpegcontext= NULL;
	AVFrame *picture;
	int i, out_size, size, outbuf_size;
	uint8_t *outbuf, *picture_buf;
	/* /libavcodec stuff */

	if(argc < 2 || argv[1] == NULL) {
		printf("Need an input filename\n");
		exit(0);
	}
	if(argc < 3 || argv[2] == NULL) {
		printf("Need an output filename\n");
		exit(0);
	}

	inputfile = aa_openinputfile(argv[1]);
	if(inputfile == NULL) {
		printf("Couldn't open inputfile: %s\n", argv[1]);
		exit(1);
	}

	outputfile = fopen(argv[2], "wb");
	if(outputfile == NULL) {
		printf("Couldn't open outputfile: %s\n", argv[2]);
		exit(1);
	}

	bufsize = sizeof(unsigned char) * inputfile->width * inputfile->height;
	textbuf = (unsigned char *)malloc(bufsize);
	attrbuf = (unsigned char *)malloc(bufsize);
	font = &aa_font16;

	/* libavcodec stuff */
	avcodec_init();
	avcodec_register_all();
	codec = avcodec_find_encoder(CODEC_ID_MPEG1VIDEO);
	if (!codec) {
		return 1;
	}

	mpegcontext= avcodec_alloc_context();
	picture= avcodec_alloc_frame();
	mpegcontext->bit_rate = 400000;
	mpegcontext->width = inputfile->width * 9;
	mpegcontext->height = inputfile->height * (font->height + 1);
	mpegcontext->frame_rate = 25;
	mpegcontext->frame_rate_base= 1;
	mpegcontext->gop_size = 30;
	mpegcontext->max_b_frames=1;

	if (avcodec_open(mpegcontext, codec) < 0) {
		return(1);
	}

	outbuf_size = 1000000;
	outbuf = (uint8_t *)malloc(outbuf_size);
	size = mpegcontext->width * mpegcontext->height;
	picture_buf = (uint8_t *)malloc((size * 3) / 2);
	picture->data[0] = picture_buf;
	picture->data[1] = picture->data[0] + size;
	picture->data[2] = picture->data[1] + size / 4;
	picture->linesize[0] = mpegcontext->width;
	picture->linesize[1] = mpegcontext->width / 2;
	picture->linesize[2] = mpegcontext->width / 2;
	/* /libavcodec stuff */

	memset((void *)picture->data[1], 128, size/4);
	memset((void *)picture->data[2], 128, size/4);

	while(0 == aa_readframe(inputfile, textbuf, attrbuf)) {
		memset((void *)picture->data[0], 16, size);
		aa_renderframeint(textbuf, attrbuf, font,
				inputfile->width, inputfile->height, picture->data[0]);
		out_size = avcodec_encode_video(mpegcontext, outbuf,
				outbuf_size, picture);
		fwrite(outbuf, 1, out_size, outputfile);
	}


	/* libavcodec stuff */
	for(; out_size; i++) {
		out_size = avcodec_encode_video(mpegcontext, outbuf, outbuf_size, NULL);
		fwrite(outbuf, 1, out_size, outputfile);
	}
	outbuf[0] = 0x00;
	outbuf[1] = 0x00;
	outbuf[2] = 0x01;
	outbuf[3] = 0xb7;
	fwrite(outbuf, 1, 4, outputfile);
	free(picture_buf);
	free(outbuf);
	free(mpegcontext);
	free(picture);
	avcodec_close(mpegcontext);
	/* /libavcodec stuff */

	aa_closeinputfile(inputfile);
	fclose(outputfile);
	return 0;
}
